@@ -65,18 +65,15 @@ def downsample(projections, spatial_factor: int, angular_factor: int):
     :param angular_factor: factor used to use less projections in angular direction
     :return: downsampled projections
     '''
-    # projections = projections[::angular_factor, ::spatial_factor, ::spatial_factor]
-    # if projections.shape[1] % spatial_factor != 0 or projections.shape[2] % spatial_factor != 0:
-    #     print('At least one of the image dimensions is not divisible by the spatial downsampling factor. This might '
-    #           'cause artifacts at the boundaries.')
-
     # using only every n-th projection (averaging in this direction does not make sense)
     projections = projections[::angular_factor, :, :]
 
     # rather than leaving out every n-th value, compute the mean value over a fixed sized window for reduction
     # projections = block_reduce(projections, block_size=(1, spatial_factor, spatial_factor), func=np.mean)
     if spatial_factor > 1:
-        projections = pyramid_reduce(projections, downscale=spatial_factor, preserve_range=True)
+        # note: keyword channel_axis was introduced in scikit image 0.19; code breaks from that version on if left
+        # unspecified
+        projections = pyramid_reduce(projections, downscale=spatial_factor, preserve_range=True, channel_axis=0)
 
     return projections
 
@@ -167,8 +164,16 @@ def read_and_preprocess_txrm(projections_file, downsample_factor=1, angular_fact
     '''
     metadata = read_metadata(str(projections_file))
     num_images = metadata['number_of_images']
-    x_shifts = metadata['x-shifts'][::angular_factor]
-    y_shifts = metadata['y-shifts'][::angular_factor]
+    x_shifts_detector = metadata['x-shifts'][::angular_factor]
+    y_shifts_detector = metadata['y-shifts'][::angular_factor]
+    x_shifts_temperature = metadata['temperature-x-shifts'][::angular_factor]
+    y_shifts_temperature = metadata['temperature-y-shifts'][::angular_factor]
+    x_shifts_amc = metadata['AMC-x-shifts'][::angular_factor]
+    y_shifts_amc = metadata['AMC-y-shifts'][::angular_factor]
+    x_shifts_source = metadata['source-x-shifts'][::angular_factor]
+    y_shifts_source = metadata['source-y-shifts'][::angular_factor]
+    x_shifts = x_shifts_detector + x_shifts_amc - x_shifts_amc[0] / 2 * np.ones_like(x_shifts_amc)
+    y_shifts = y_shifts_detector + y_shifts_amc - y_shifts_amc[0] / 2 * np.ones_like(y_shifts_amc)
     reference = metadata['reference']
     if not dose_reduction == 1.:
         reference_smooth = smooth_reference(reference)
