@@ -366,6 +366,7 @@ def read_metadata(file_name, slice_range=None):
     if reference is not None:
         metadata['reference'] = reference[slice_range[1:]]
 
+
     ole.close()
 
     return metadata
@@ -413,69 +414,113 @@ def read_ole_metadata(ole):
     """
 
     number_of_images = _read_ole_value(ole, "ImageInfo/NoOfImages", "<I")
+    multi_reference_found = False
+    single_reference_found = False
+    for entry in ole.listdir():
+        if 'MultiReferenceData' in entry:
+            multi_reference_found = True
+        if 'ReferenceData' in entry:
+            single_reference_found = True
+    reference_prefix = None
+    read_reference_info = True
+    if single_reference_found and multi_reference_found:
+        # print('\nFound both single- and multi-reference data. Using single.\n')
+        reference_prefix = 'ReferenceData'
+    elif single_reference_found and not multi_reference_found:
+        reference_prefix = 'ReferenceData'
+    elif multi_reference_found and not single_reference_found:
+        # print('\nFound multi-reference data. Using mean of all reference images.\n')
+        reference_prefix = 'MultiReferenceData'
+    else:
+        print('\nCould not find any reference data in metadata.\n')
+        read_reference_info = False
 
-    metadata = {
-        'facility': _read_ole_value(ole, 'SampleInfo/Facility', '<50s'),
-        'image_width': _read_ole_value(ole, 'ImageInfo/ImageWidth', '<I'),
-        'image_height': _read_ole_value(ole, 'ImageInfo/ImageHeight', '<I'),
-        'data_type': _read_ole_value(ole, 'ImageInfo/DataType', '<1I'),
-        'number_of_images': number_of_images,
-        'pixel_size': _read_ole_value(ole, 'ImageInfo/pixelsize', '<f'),
-        'cam_pixel_size': _read_ole_value(ole, 'ImageInfo/CamPixelSize', '<f'),
-        #'reference_filename': _read_ole_value(ole, 'ImageInfo/referencefile', '<260s'),
-        'reference_exposure_time': _read_ole_value(ole, 'ReferenceData/ExpTime', '<f'),
-        'reference_current': _read_ole_value(ole, 'ReferenceData/XrayCurrent', '<f'),
-        'reference_voltage': _read_ole_value(ole, 'ReferenceData/XrayVoltage', '<f'),
-        'reference_data_type': _read_ole_value(ole, 'referencedata/DataType', '<1I'),
-        'image_data_type': _read_ole_value(ole, 'ImageInfo/DataType', '<1I'),
-        # NOTE: converting theta to radians from degrees
-        'thetas': _read_ole_arr(ole, 'ImageInfo/Angles', "<{0}f".format(number_of_images)) * np.pi / 180.,
-        'x_positions': _read_ole_arr(ole, 'ImageInfo/XPosition', "<{0}f".format(number_of_images)),
-        'y_positions': _read_ole_arr(ole, 'ImageInfo/YPosition', "<{0}f".format(number_of_images)),
-        'z_positions': _read_ole_arr(ole, 'ImageInfo/ZPosition', "<{0}f".format(number_of_images)),
-        'x-shifts': _read_ole_arr(ole, 'alignment/x-shifts', "<{0}f".format(number_of_images)),
-        'y-shifts': _read_ole_arr(ole, 'alignment/y-shifts', "<{0}f".format(number_of_images)),
-        'center_shift': _read_ole_value(ole, 'ReconSettings/CenterShift', '<f'),
-        'rotation_angle': _read_ole_value(ole, 'ReconSettings/RotationAngle', '<f'),
-        'source_isocenter_distance': _read_ole_value(ole, 'ImageInfo/StoRADistance', "<{0}f".format(number_of_images)),
-        'detector_isocenter_distance': _read_ole_value(ole, 'ImageInfo/DtoRADistance', "<{0}f".format(number_of_images)),
-        'cone_angle': _read_ole_value(ole, 'ImageInfo/ConeAngle', "<{0}f".format(number_of_images)),
-        'fan_angle': _read_ole_value(ole, 'ImageInfo/FanAngle', "<{0}f".format(number_of_images)),
-        'camera_offset': _read_ole_value(ole, 'ImageInfo/CameraOffset', '<f'),
-        'source_drift': _read_ole_value(ole, 'ImageInfo/SourceDriftTotal', '<f'),
-        'current': _read_ole_value(ole, 'ImageInfo/Current', '<f'),
-        'voltage': _read_ole_value(ole, 'ImageInfo/Voltage', '<f'),
-        'power': _read_ole_value(ole, 'AcquisitionSettings/SrcPower', '<f'),
-        'exposure_time': _read_ole_value(ole, 'AcquisitionSettings/ExpTime', '<f'),
-        'binning': _read_ole_value(ole, 'AcquisitionSettings/Binning', '<I'),
-        'filter': _read_ole_value(ole, 'AcquisitionSettings/SourceFilterName', '<260s')#.decode('utf-8').rstrip('\x00')
-        #'temperatures': _read_ole_arr(ole, 'TemperatureInfo/Temperatures', "<{0}f".format(4 * number_of_images)),
-        #'amc_drift_x_shifts': _read_ole_arr(ole, 'Alignment/AMCDriftXShifts', "<{0}f".format(number_of_images)),
-        #'amc_drift_y_shifts': _read_ole_arr(ole, 'Alignment/AMCDriftYShifts', "<{0}f".format(number_of_images)),
-        #'amc_drift_x_shifts2': _read_ole_arr(ole, 'AMC/Alignment/AMCDriftXShifts', "<{0}d".format(number_of_images)),
-        #'amc_drift_y_shifts2': _read_ole_arr(ole, 'AMC/Alignment/AMCDriftYShifts', "<{0}d".format(number_of_images)),
-        #'align_mode': _read_ole_value(ole, 'Alignment/AlignMode', '<I'),
-        #'encoder_shifts_applied': _read_ole_value(ole, 'Alignment/EncoderShiftsApplied', '<I'),
-        #'encoder_x_shifts': _read_ole_arr(ole, 'Alignment/EncoderXShifts', "<{0}f".format(number_of_images)),
-        #'encoder_y_shifts': _read_ole_arr(ole, 'Alignment/EncoderYShifts', "<{0}f".format(number_of_images)),
-        #'metrology_shifts_applied': _read_ole_value(ole, 'Alignment/MetrologyShiftsApplied', '<I'),
-        #'reference_shifts_applied': _read_ole_value(ole, 'Alignment/ReferenceShiftsApplied', '<I'),
-        #'source_drift_x_shifts': _read_ole_arr(ole, 'Alignment/SourceDriftXShifts', "<{0}f".format(number_of_images)),
-        #'source_drift_y_shifts': _read_ole_arr(ole, 'Alignment/SourceDriftYShifts', "<{0}f".format(number_of_images)),
-        #'source_spot_shifts_applied': _read_ole_value(ole, 'Alignment/SourceSpotShiftsApplied', '<I'),
-        #'source_spot_x_shift': _read_ole_arr(ole, 'Alignment/SourceSpotXShift', "<{0}f".format(number_of_images)),
-        #'source_spot_y_shift': _read_ole_arr(ole, 'Alignment/SourceSpotYShift', "<{0}f".format(number_of_images)),
-        #'spot_shifts_applied': _read_ole_value(ole, 'Alignment/SpotShiftsApplied', '<I'),
-        #'spot_x_shifts': _read_ole_arr(ole, 'Alignment/SpotXShifts', "<{0}f".format(number_of_images)),
-        #'spot_y_shifts': _read_ole_arr(ole, 'Alignment/SpotYShifts', "<{0}f".format(number_of_images)),
-        #'stage_shifts_applied': _read_ole_value(ole, 'Alignment/StageShiftsApplied', '<I'),
-        #'temperature_x_shifts': _read_ole_arr(ole, 'Alignment/TemperatureXShifts', "<{0}f".format(number_of_images)),
-        #'temperature_y_shifts': _read_ole_arr(ole, 'Alignment/TemperatureYShifts', "<{0}f".format(number_of_images)),
-        #'use_sample_or_src_drift': _read_ole_value(ole, 'Alignment/UseSampleOrSrcDrift', '<I'),
-        #'user_shifts_applied': _read_ole_value(ole, 'Alignment/UserShiftsApplied', '<I'),
-        #'absorption_scale_factor': _read_ole_value(ole, 'ImageInfo/AbsorptionScaleFactor', '<f'),
-        #'absorption_scale_offset': _read_ole_value(ole, 'ImageInfo/AbsorptionScaleOffset', '<f')
-    }
+    if read_reference_info:
+        metadata = {
+            'facility': _read_ole_value(ole, 'SampleInfo/Facility', '<50s'),
+            'image_width': _read_ole_value(ole, 'ImageInfo/ImageWidth', '<I'),
+            'image_height': _read_ole_value(ole, 'ImageInfo/ImageHeight', '<I'),
+            'data_type': _read_ole_value(ole, 'ImageInfo/DataType', '<1I'),
+            'number_of_images': number_of_images,
+            'pixel_size': _read_ole_value(ole, 'ImageInfo/pixelsize', '<f'),
+            # 'cam_pixel_size': _read_ole_value(ole, 'ImageInfo/CamPixelSize', '<f'),
+            'reference_exposure_time': _read_ole_value(ole, reference_prefix + '/ExpTime', '<f'),
+            'reference_current': _read_ole_value(ole, reference_prefix + '/XrayCurrent', '<f'),
+            'reference_voltage': _read_ole_value(ole, reference_prefix + '/XrayVoltage', '<f'),
+            'reference_data_type': _read_ole_value(ole, reference_prefix + '/DataType', '<1I'),
+            'image_data_type': _read_ole_value(ole, 'ImageInfo/DataType', '<1I'),
+            # NOTE: converting theta to radians from degrees
+            'thetas': _read_ole_arr(ole, 'ImageInfo/Angles', "<{0}f".format(number_of_images)) * np.pi / 180.,
+            # 'x_positions': _read_ole_arr(ole, 'ImageInfo/XPosition', "<{0}f".format(number_of_images)),
+            # 'y_positions': _read_ole_arr(ole, 'ImageInfo/YPosition', "<{0}f".format(number_of_images)),
+            # 'z_positions': _read_ole_arr(ole, 'ImageInfo/ZPosition', "<{0}f".format(number_of_images)),
+            'x-shifts': _read_ole_arr(ole, 'alignment/x-shifts', "<{0}f".format(number_of_images)),
+            'y-shifts': _read_ole_arr(ole, 'alignment/y-shifts', "<{0}f".format(number_of_images)),
+            'AMC-x-shifts': _read_ole_arr(ole, 'alignment/AMCDriftXShifts', "<{0}f".format(number_of_images)),
+            'AMC-y-shifts': _read_ole_arr(ole, 'alignment/AMCDriftYShifts', "<{0}f".format(number_of_images)),
+            'temperature-x-shifts': _read_ole_arr(ole, 'alignment/TemperatureXShifts', "<{0}f".format(number_of_images)),
+            'temperature-y-shifts': _read_ole_arr(ole, 'alignment/TemperatureYShifts', "<{0}f".format(number_of_images)),
+            'source-x-shifts': _read_ole_arr(ole, 'alignment/SourceDriftXShifts', "<{0}f".format(number_of_images)),
+            'source-y-shifts': _read_ole_arr(ole, 'alignment/SourceDriftYShifts', "<{0}f".format(number_of_images)),
+            'align-mode': _read_ole_arr(ole, 'alignment/AlignMode', '<1I'),
+            'center_shift': _read_ole_value(ole, 'ReconSettings/CenterShift', '<f'),
+            'rotation_angle': _read_ole_value(ole, 'ReconSettings/RotationAngle', '<f'),
+            'source_isocenter_distance': _read_ole_value(ole, 'ImageInfo/StoRADistance', "<{0}f".format(number_of_images)),
+            'detector_isocenter_distance': _read_ole_value(ole, 'ImageInfo/DtoRADistance', "<{0}f".format(number_of_images)),
+            'cone_angle': _read_ole_value(ole, 'ImageInfo/ConeAngle', "<{0}f".format(number_of_images)),
+            'fan_angle': _read_ole_value(ole, 'ImageInfo/FanAngle', "<{0}f".format(number_of_images)),
+            'camera_offset': _read_ole_value(ole, 'ImageInfo/CameraOffset', '<f'),
+            'source_drift': _read_ole_value(ole, 'ImageInfo/SourceDriftTotal', '<f'),
+            'current': _read_ole_value(ole, 'ImageInfo/Current', '<f'),
+            'voltage': _read_ole_value(ole, 'ImageInfo/Voltage', '<f'),
+            'power': _read_ole_value(ole, 'AcquisitionSettings/SrcPower', '<f'),
+            'exposure_time': _read_ole_value(ole, 'AcquisitionSettings/ExpTime', '<f'),
+            'binning': _read_ole_value(ole, 'AcquisitionSettings/Binning', '<I'),
+            'filter': _read_ole_value(ole, 'AcquisitionSettings/SourceFilterName', '<260s')
+        }
+    else:
+        metadata = {
+            'facility': _read_ole_value(ole, 'SampleInfo/Facility', '<50s'),
+            'image_width': _read_ole_value(ole, 'ImageInfo/ImageWidth', '<I'),
+            'image_height': _read_ole_value(ole, 'ImageInfo/ImageHeight', '<I'),
+            'data_type': _read_ole_value(ole, 'ImageInfo/DataType', '<1I'),
+            'number_of_images': number_of_images,
+            'pixel_size': _read_ole_value(ole, 'ImageInfo/pixelsize', '<f'),
+            # 'cam_pixel_size': _read_ole_value(ole, 'ImageInfo/CamPixelSize', '<f'),
+            'image_data_type': _read_ole_value(ole, 'ImageInfo/DataType', '<1I'),
+            # NOTE: converting theta to radians from degrees
+            'thetas': _read_ole_arr(ole, 'ImageInfo/Angles', "<{0}f".format(number_of_images)) * np.pi / 180.,
+            # 'x_positions': _read_ole_arr(ole, 'ImageInfo/XPosition', "<{0}f".format(number_of_images)),
+            # 'y_positions': _read_ole_arr(ole, 'ImageInfo/YPosition', "<{0}f".format(number_of_images)),
+            # 'z_positions': _read_ole_arr(ole, 'ImageInfo/ZPosition', "<{0}f".format(number_of_images)),
+            'x-shifts': _read_ole_arr(ole, 'alignment/x-shifts', "<{0}f".format(number_of_images)),
+            'y-shifts': _read_ole_arr(ole, 'alignment/y-shifts', "<{0}f".format(number_of_images)),
+            'AMC-x-shifts': _read_ole_arr(ole, 'alignment/AMCDriftXShifts', "<{0}f".format(number_of_images)),
+            'AMC-y-shifts': _read_ole_arr(ole, 'alignment/AMCDriftYShifts', "<{0}f".format(number_of_images)),
+            'temperature-x-shifts': _read_ole_arr(ole, 'alignment/TemperatureXShifts',
+                                                  "<{0}f".format(number_of_images)),
+            'temperature-y-shifts': _read_ole_arr(ole, 'alignment/TemperatureYShifts',
+                                                  "<{0}f".format(number_of_images)),
+            'source-x-shifts': _read_ole_arr(ole, 'alignment/SourceDriftXShifts', "<{0}f".format(number_of_images)),
+            'source-y-shifts': _read_ole_arr(ole, 'alignment/SourceDriftYShifts', "<{0}f".format(number_of_images)),
+            'align-mode': _read_ole_arr(ole, 'alignment/AlignMode', '<1I'),
+            'center_shift': _read_ole_value(ole, 'ReconSettings/CenterShift', '<f'),
+            'rotation_angle': _read_ole_value(ole, 'ReconSettings/RotationAngle', '<f'),
+            'source_isocenter_distance': _read_ole_value(ole, 'ImageInfo/StoRADistance',
+                                                         "<{0}f".format(number_of_images)),
+            'detector_isocenter_distance': _read_ole_value(ole, 'ImageInfo/DtoRADistance',
+                                                           "<{0}f".format(number_of_images)),
+            'cone_angle': _read_ole_value(ole, 'ImageInfo/ConeAngle', "<{0}f".format(number_of_images)),
+            'fan_angle': _read_ole_value(ole, 'ImageInfo/FanAngle', "<{0}f".format(number_of_images)),
+            'camera_offset': _read_ole_value(ole, 'ImageInfo/CameraOffset', '<f'),
+            'source_drift': _read_ole_value(ole, 'ImageInfo/SourceDriftTotal', '<f'),
+            'current': _read_ole_value(ole, 'ImageInfo/Current', '<f'),
+            'voltage': _read_ole_value(ole, 'ImageInfo/Voltage', '<f'),
+            'power': _read_ole_value(ole, 'AcquisitionSettings/SrcPower', '<f'),
+            'exposure_time': _read_ole_value(ole, 'AcquisitionSettings/ExpTime', '<f'),
+            'binning': _read_ole_value(ole, 'AcquisitionSettings/Binning', '<I'),
+            'filter': _read_ole_value(ole, 'AcquisitionSettings/SourceFilterName', '<260s')
+        }
 
     try:
         metadata['filter'] = metadata['filter'].decode('utf-8').rstrip('\x00')
@@ -490,8 +535,19 @@ def read_ole_metadata(ole):
                 reference_filename = reference_filename[:i]
                 break
     #metadata['reference_filename'] = reference_filename
-    if ole.exists('referencedata/image'):
-        reference = _read_ole_image(ole, 'referencedata/image', metadata, metadata['reference_data_type'])
+    if reference_prefix == 'ReferenceData':
+        if ole.exists(reference_prefix + '/Image'):
+            reference = _read_ole_image(ole, reference_prefix + '/Image', metadata, metadata['reference_data_type'])
+        else:
+            reference = None
+    elif reference_prefix == 'MultiReferenceData':
+        i = 1
+        reference = []
+        while ole.exists(reference_prefix + '/Image' + str(i)):
+            reference.append(_read_ole_image(ole, reference_prefix + '/Image' + str(i), metadata, metadata['reference_data_type']))
+            i += 1
+        reference = np.array(reference)
+        reference = np.mean(reference, axis=0)
     else:
         reference = None
     metadata['reference'] = reference
@@ -682,3 +738,44 @@ def _read_ole_image(ole, label, metadata, datatype=None):
     )
     return image
 
+
+def read_amc_images(file_name, slice_range=None, to_list=False):
+    file_name = _check_read(file_name)
+    try:
+        ole = olefile.OleFileIO(file_name)
+    except IOError:
+        print('No such file or directory: %s', file_name)
+        return False
+
+    metadata = read_ole_metadata(ole)
+
+    if to_list:
+        array_of_images = []
+    else:
+        array_of_images = np.empty(
+            _shape_after_slice(
+                (
+                    21,
+                    metadata["image_height"],
+                    metadata["image_width"],
+                ),
+                slice_range
+            ),
+            dtype=np.float32
+        )
+
+    if slice_range is None:
+        slice_range = (slice(None), slice(None), slice(None))
+    else:
+        slice_range = _make_slice_object_a_tuple(slice_range)
+
+    for i, idx in enumerate(range(*slice_range[0].indices(21))):
+        img_string = "AMC/ImageData1/Image{}".format(int(idx + 1))
+        if to_list:
+            array_of_images.append(_read_ole_image(ole, img_string, metadata)[slice_range[1:]])
+        else:
+            array_of_images[i] = _read_ole_image(ole, img_string, metadata)[slice_range[1:]]
+
+    ole.close()
+
+    return array_of_images
